# NAME:
#	dnsmagic.mk - make agnostic rules for DNSMagic
#
# DESCRIPTION:
#
#	This makefile is intended to be make agnostic, so we avoid
#	syntax which is not supported by both BSD make and GNU make.
#	It is intended to be included by Makefile or GNUmakefile.
#
#	If your make(1) does not support 'include'ing files, then
#	remove the -N option from 'dnsdeps' and it will append
#	.depend to the makefile.
#
#	The setup here relies on SOA records being kept in files named
#	zone.soa.  We use zone.rev for in-addr.arpa data files and
#	zone.db for others.  If a .soa file is needed and is missing,
#	we run 'updsoa.pl' to create it.
#
#	'make depend' will ensure that all the primary zone files
#	referenced by named.boot or named.conf are recursively checked 
#	for dependencies.
#
#	'make' will update the serial number of any .soa files which
#	are out of date.
#
#	'make regress' will run our regression suite.
#
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

# RCSid:
#	$Id: dnsmagic.mk,v 1.5 2016/04/11 00:38:00 sjg Exp $
#
#	@(#)Copyright (c) 1995-2016 Simon J. Gerraty.
#	All rights reserved.
#	
#	This software is the property of Simon J. Gerraty.
#	It may be used only in accordance with the conditions outlined
#	in the accompanying LICENSE file. If your distribution does
#	not contain a LICENSE file, then you have no rights to use
#	this software and should contact sales@crufty.net to obtain a
#	licensed distribution.
#	
#	This software is provided "AS IS" in the hope that it will be
#	of use.  Since it is not possible to anticipate all possible
#	use of this software, there is absolutely NO WARRANTY.
#
#	Please forward bug reports and fixes to help@crufty.net
#

# since we often run from cron(8), ensure we have
# what we want in the path...
SHARE?=/usr/local/share
DNSMAGIC?=${SHARE}/dns

BIN=PATH=./bin:${DNSMAGIC}/bin:${SHARE}/bin:$$PATH:/etc/rc_d

UPDSOA=updsoa
# lose the -N if you need the dependencies appended to makefile.
# better yet, get bmake
MKDEP=dnsdeps -N
BOUNCE=bouncedns
RELOAD=bouncedns reload
DNS_REGRESS=${DNSMAGIC}/regress -v
UPDREV=updrev -N ${PZONES} ${UPDREV_OPTS} ${NDDUMP}
UPREV_DEPS=${NDDUMP}
# An SCM agnostic update mechanism
SCM_UPDATE?= scm-update.sh

# probably best not to do this here, as make(1)
# has already built a dependency graph, so if this
# does update anything, make needs to be re-run anyway.
#_SCM=.scm

all:	${_SCM} .depend prezones zones ${NDBOOT_SEC} ${NDBOOT}.test

# this lets us invoke updsoa and bounce once only
prezones:
	@rm -f .zones
	@[ -d secondary ] || mkdir secondary

zones:	.zones
	@test -s .zones && ${BIN} xargs ${UPDSOA} < .zones || :
	@test -s .zones && { echo ${RELOAD}; DNSDIR=. ${BIN} ${RELOAD} > /dev/null; } || :

# .zones is actually contained in .depend, 
# the .zones below is just to keep make quiet
# when .depend does not yet exist
.zones:
.depend depend:	${NDBOOT} ${PZONES} ${PZONE_DEPS}
	@echo ${MKDEP} ${NDBOOT}
	@${BIN} ${MKDEP} ${NDBOOT}

# we don't use the usual rules
.SUFFIXES:

# edit this to match what you use - but you'll have to fix bin/* too.
.SUFFIXES: .db .rev .soa .boot .boot.sec .boot.test .conf .conf.sec .conf.test

.db.soa .rev.soa:
	@echo '${UPDSOA} $@'; echo $@ >> .zones

.boot.boot.sec:
	@echo '${UPD_SNDBOOT} $? > $@ || rm -f $@'
	@${BIN} ${UPD_SNDBOOT} $? > $@ || rm -f $@
	@echo ${BOUNCE}; DNSDIR=. ${BIN} ${BOUNCE} > /dev/null

# this is for regression testing
.boot.boot.test:
	grep -v -i "^second" $? | sed "s,${LIVE_DNSDIR},`pwd`," > $@

# for secondary servers
.conf.conf.sec:
	sed -e 's/primary.zones/slave.zones/' ${CONF_SEC_SED} < $? > $@
	@echo ${BOUNCE}; DNSDIR=. ${BIN} ${BOUNCE} > /dev/null

slave.zones:	primary.zones
	@echo '${SLAVEZONES} ${DNS_MASTERS} < $? > $@ || rm -f $@'
	@${BIN} ${SLAVEZONES} ${DNS_MASTERS} < $? > $@ || rm -f $@
	@echo ${BOUNCE}; DNSDIR=. ${BIN} ${BOUNCE} > /dev/null

# we need to reload when secondary.zones changes too.
.seconds:	secondary.zones
	@echo ${BOUNCE}; DNSDIR=. ${BIN}  ${RELOAD} > /dev/null
	@touch $@

.conf.conf.test:
	sed ${CONF_TEST_SED} -e '/query-source/d' \
	-e '/listen-on/d' -e '/pid-file/d' \
	-e 's,^[ 	]*directory.*,listen-on port ${ND_PORT} { 127.0.0.1; }; pid-file "./named.pid";,' \
	-e '/secondary/d' $? > $@

regress .regress.ests:	.depend ${NDBOOT}.test
	${BIN} ${DNS_REGRESS}

# this is not an automagic target as not everyone wants it.
revs:	.regress.ests ${UPDREV_DEPS}
	@echo ${UPDREV}
	@${BIN} ${UPDREV}

.scm:
	@echo ${SCM_UPDATE}; test -f .noscm || ${SCM_UPDATE}

all: .dnsmagicrc
.dnsmagicrc:
	@echo DNSMAGIC=${DNSMAGIC} > $@
	@test ! -d .git || ln -sf ${DNSMAGIC}/pre-commit.sh .git/hooks/pre-commit
	@test ! -d .hg || { echo '[hooks]'; echo precommit = ${DNSMAGIC}/pre-commit.sh; } >> .hg/hgrc
